#!/bin/ksh

# -------------------------------------------------------------------------------------------
# Usage:
#
#     enterprise.sh install <base install dir>
#     enterprise.sh uninstall
#
# NOTE: Requires kron shell
# -------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------
# Validate the input
# -------------------------------------------------------------------------------------------
shname=`basename $0`

if [ -z "$1" ]; then
   echo "Usage is:"
   echo "      ${shname} install <base directory>"
   echo "           '${shname} install /opt/softech' will install into /opt/softech/mwcore"
   echo "      ${shname} uninstall"
   echo "ERROR: No action specified"
   exit 1
fi

if [ "$1" != "install" ]; then
   if [ "$1" != "uninstall" ]; then
      echo "Usage is:"
      echo "      ${shname} install <base directory>"
      echo "           ${shname} install /opt/softech will install into /opt/softech/mwcore"
      echo "           The install base must already exist"
      echo "      ${shname} uninstall"
      echo "ERROR: Action non install nor uninstall: $1"
      exit 1
   fi
fi

instdir=$2

if [ "$1" = "install" ]; then
   if [ "$instdir" = "" ]; then
      if [ ! -z "$PSTHOME" ]; then
         echo "Usage is:"
         echo "      ${shname} install ~<base directory~>"
         echo "ERROR: <base directory> was not entered, would you like me to install into"
         echo "       $PSTHOME/mwcore ?"
         echo "Continue? (y/n + ENTER)"
         read answer
         if [ "$answer" = "n" ]; then
            exit 1
         fi
         if [ "$answer" != "y" ]; then
            echo Answer was not y or n
            exit 1
         fi
         instdir=$PSTHOME
      else
         echo "Usage is:"
         echo "      ${shname} install <base directory>"
         echo "           ${shname} install /opt/softech will install into /opt/softech/mwcore"
         echo "           The install base must already exist"
         echo "      ${shname} uninstall"
         echo "ERROR: Action is install, missing base directory"
         exit 1
      fi
   fi
   if [ ! -d $instdir ]; then
      echo "Usage is:"
      echo "      ${shname} install <base directory>"
      echo "           ${shname} install /opt will install into /opt/softech/mwcore"
      echo "           The install base must already exist"
      echo "      ${shname} uninstall"
      echo "ERROR: Action is install, base directory does not exist: $instdir"
      exit 1
   fi
fi

# -------------------------------------------------------------------------------------------
# Must be root since we're going to get into the init.d processing
# -------------------------------------------------------------------------------------------

# Get euid from 'id' since it will handle su (who am i returns logon id) id returns uid=xxx(name) gid=xxx(group)
id=`id | awk -F= '{print $2}' | awk -F"(" '{print $1}'`

# Must be root
if [ $id -ne 0 ]; then
   id
   echo "ERROR: You must be root to $1 the Enterprise Mode Core package"
   exit 1
fi

# -------------------------------------------------------------------------------------------
# We have to have PSTHOME set, if not see if its in the directory below us (assume we're
# being executed from optim-install-dir/bin)
# -------------------------------------------------------------------------------------------
if [ -z "$PSTHOME" ]; then

   if [ -f ../rtsetenv ]; then
      . ../rtsetenv
   else
      thisDir=`dirname $0`
      if [ -f $thisDir/../rtssetenv ]; then
         . $thisDir/../rtsetenv
      fi
   fi

   if [ -z "$PSTHOME" ]; then
      echo "ERROR: Please set environment variable PSTHOME to the install directory of Optim"
      exit 1
   fi

fi

if [ ! -d $PSTHOME ]; then
   echo "ERROR: Please set environment variable PSTHOME to the install"
   echo "       directory of Optim or make sure you've fixed up 'rtsetenv'"
   echo "       per Optim install instructions"
   exit 1
fi

# -------------------------------------------------------------------------------------------
# Setup mainwin paths since we're going to use their utility
# -------------------------------------------------------------------------------------------
if [ -z "$MWSETUP_GUARD" ]; then
   export MWHOME=$PSTHOME/mw
   export MWDEBUG=1
   . $MWHOME/setmwruntime
   unset MWDEBUG
fi

# -------------------------------------------------------------------------------------------
# Insure core services are NOT running
# -------------------------------------------------------------------------------------------
status=`mwadm status | awk '/status:/ {print $4}'`
if [ "$status" = "running" ]; then
   echo "Stopping existing core services"
   mwadm stop -f
   status=`mwadm status | awk '/status:/ {print $4}'`
   if [ "$status" == "running" ]; then
      echo "ERROR: Failed to stop core services!"
      echo "       Please execute 'mwadm stop' prior to re-executing $shname"
      exit 1
   fi
fi

# -------------------------------------------------------------------------------------------
# If I don't get rid of the MWREGISTRY then Enterprise mode will think I'm trying to
# override it (probably to this user's home directory)
# -------------------------------------------------------------------------------------------
unset MWREGISTRY

# -------------------------------------------------------------------------------------------
# uninstall
# -------------------------------------------------------------------------------------------
if [ "$1" = "uninstall" ]; then

   # Make sure its installed and get the install directory
   if [ ! -r /etc/mainwin.conf ]; then
      echo "ERROR: Enterprise Core Services are not installed, /etc/mainwin.conf does not exist"
      exit 1
   fi

   # Get the install directory
   instdir=`grep CORE_PATH /etc/mainwin.conf | awk -F= '{print $2}'`

   if [ -z "${instdir}" ]; then
      echo "ERROR: Enterprise Core Services config file /etc/mainwin.conf corrupt, no CORE_PATH entry"
      exit 1
   fi

   # Change to the the core's bin directory (mw/bin)
   cd ${instdir}/mw/bin

   # Insure directory is somewhat intact
   if [ ! -f uninstall.mwcore ]; then
      echo "ERROR: Could not find the uninstall script ${instdir}/mw/bin/uninstall.mwcore"
      exit 1
   fi


   # Uninstall (it will ask if its ok)
   echo "Uninstalling Enterprise Core in ${instdir}"
   ./uninstall.mwcore

   # Make sure enterprise is the default mode (allow this to find it on the path so it changes the correct default file)
   # NOTE: Use the mwset_rtmode distributed with the product since we just deleted core services copy
   echo "Setting Professional mode as the default mode"
   $PSTHOME/mw/bin/mwset_rtmode -s -p

   exit $?

fi

# -------------------------------------------------------------------------------------------
# Make sure install directory (CORE_SERVICES) exist.
# We distribute core.tar containing the core services, it un-tars to that directory.
# If we're restarting this install then the work directory CORE_SERVICES may still be
# present, if so save time and use it.
# -------------------------------------------------------------------------------------------

# Either the tar or the directory must exist
if [ ! -f $PSTHOME/mw/core.tar ]; then
   if [ ! -d $PSTHOME/mw/CORE_SERVICES ]; then
      echo "ERROR: You must have obtained the Enterprise Core tar file (core.tar)"
      echo "       prior to running $shname and copied it into the"
      echo "       $PSTHOME/mw directory"
      exit 1
   fi
fi

# If CORE_SERVICES does not exist we have to create it now, from either core.tar or unzip core.tar.gz first
if [ ! -d $PSTHOME/mw/CORE_SERVICES ]; then

   cd $PSTHOME/mw

   # Should not occur at this point...
   if [ ! -f core.tar ]; then
      echo "ERROR: Unexpected error, $PSTHOME/mw/core.tar does not exist all of a sudden!"
      exit 1
   fi

   # Create the core services install directory
   echo "Creating enterprise mode install directory"
   tar -xpf core.tar

fi

# -------------------------------------------------------------------------------------------
# Everything looks good, ok to install?
# -------------------------------------------------------------------------------------------
echo "Installing Enterprise Mode Core files into $instdir/mwcore directory"
echo "Continue? (y/n + ENTER)"
read answer

if [ "$answer" = "n" ]; then
   exit 0
fi

if [ "$answer" != "y" ]; then
   echo Answer was not y or n
   exit 0
fi

# -------------------------------------------------------------------------------------------
# Install it
# -------------------------------------------------------------------------------------------

# Change to the core services install directory
cd $PSTHOME/mw/CORE_SERVICES/MSC

# Execute the install with assumptions
echo "Installing Enterprise Core files into $instdir/mwcore"
./install -instdir $instdir/mwcore -datadir $instdir/mwcore/mwcoredata -adm root

# -------------------------------------------------------------------------------------------
# Validate install
# -------------------------------------------------------------------------------------------
echo "Validating install..."

# Make sure enterprise mode config exists
if [ ! -r /etc/mainwin.conf ]; then
   echo "ERROR: Enterprise Core Services failed to properly install, missing /etc/mainwin.conf file"
   exit 1
fi

# And the core directory exists
if [ ! -d $instdir/mwcore ]; then
   echo "ERROR: Enterprise Core Services failed to properly install, missing $instdir/mwcore directory"
   exit 1
fi

# And the core data directory exists
if [ ! -d $instdir/mwcore/mwcoredata ]; then
   echo "ERROR: Enterprise Core Services failed to properly install, missing $instdir/mwcore/mwcoredata directory"
   exit 1
fi

# And the core data bin directory exists
if [ ! -d $instdir/mwcore/mw ]; then
   echo "ERROR: Enterprise Core Services failed to properly install, missing $instdir/mwcore/mw directory"
   exit 1
fi

# See if init.d files exist, depends on platform
case `uname -s` in

   SunOS)
       # Script in 2nd level init.d processing
       if [ ! -f /etc/rc2.d/S99mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /etc/rc2.d/S99mwcore_serivces"
          exit 1
       fi
       ;;

   HP-UX)
       # init.d time script
       if [ ! -f /sbin/init.d/mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /sbin/init.d/mwcore_serivces"
          exit 1
       fi
       # And this is a symbolic link to it that determines to do it in level 3
       if [ ! -L /sbin/rc3.d/S99mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /sbin/rc3.d/S99mwcore_serivces"
          exit 1
       fi
       ;;

   AIX)
       # Script in init.d processing
       if [ ! -f /etc/rc.mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /etc/rc.mwcore_serivces"
          exit 1
       fi
       # Check if its in the inittab file (the legal way)
       /usr/sbin/lsitab "mwcore" > /dev/null 2>&1
       if [ $? -ne 0 ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, 'mwcore' not in /etc/inittab"
          exit 1
       fi
       ;;

   Linux)
       # This is the init.d file
       if [ ! -f /etc/rc.d/init.d/mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /etc/rc.d/init.d/mwcore_serivces"
          exit 1
       fi
       # And this is a symbolic link to it that determines to do it in level 35
       if [ ! -L /etc/rc.d/rc[35].d/S99mwcore_services ]; then
          echo "ERROR: Enterprise Core Services failed to properly install, missing /etc/rc.d/rc3.d/S99mwcore_services"
          exit 1
       fi
       ;;

   *)
       echo Unknown O/S
       exit 1
       ;;

esac

# -------------------------------------------------------------------------------------------
# Clean up, we let this directory hang around till now to make retry of the install faster
# -------------------------------------------------------------------------------------------
echo "Deleting work directory $PSTHOME/mw/CORE_SERVICES"
cd $PSTHOME/mw
rm -r CORE_SERVICES > /dev/null 2>&1

# -------------------------------------------------------------------------------------------
# Set default mode to Enterprise and get rid of any override MWRT_MODE
# -------------------------------------------------------------------------------------------
echo "Setting Enterprise mode as the default mode"
unset MWRT_MODE
$PSTHOME/mw/bin/mwset_rtmode -s -e

# -------------------------------------------------------------------------------------------
# Make sure it's running
# -------------------------------------------------------------------------------------------
echo "Starting Enterprise Core Services"
status=`mwadm status | awk '/status:/ {print $4}'`
if [ "$status" != "running" ]; then
   mwadm start > /dev/null 2>&1
   status=`mwadm status | awk '/status:/ {print $4}'`
   if [ "$status" != "running" ]; then
      echo "ERROR: Failed to start Enterprise Mode Kernel programs (watchdog, mwrpcss, regss), use 'mwadm status'"
      exit 1
   fi
fi

# make sure its running Enterprise
mode=`mwadm status | awk '/Run-time/ {print $4}'`
if [ "$mode" != "enterprise" ]; then
   echo "ERROR: Failed to start Enterprise Mode, still running $mode mode"
   exit 1
fi

# -------------------------------------------------------------------------------------------
# Allow SVCE, not running as root, to modify local machine hive sections
# -------------------------------------------------------------------------------------------
echo "Modify local machine hive permissions"
mwadm reg -chmod -r 666 'HKEY_LOCAL_MACHINE\\System'
mwadm reg -chmod -r 666 'HKEY_LOCAL_MACHINE\\Software'

state=$(mwadm reg -ls 'HKEY_LOCAL_MACHINE\\System' | awk '{print $1}')

if [ "$state" != "krw-rw-rw-" ]; then
   echo "ERROR: Failed to set write access to the System secton of the local hive"
   exit 1
fi

state=$(mwadm reg -ls 'HKEY_LOCAL_MACHINE\\Software' | awk '{print $1}')

if [ "$state" != "krw-rw-rw-" ]; then
   echo "ERROR: Failed to set write access to the Software secton of the local hive"
   exit 1
fi

# -------------------------------------------------------------------------------------------
# Show everything ok
# -------------------------------------------------------------------------------------------
echo "Enterprise mode successfully installed and started and will start on each IPL"

exit 0

